package cn.zhangdeman;

import cn.zhangdeman.annotation.CustomExceptionScanAnnotation;
import org.springframework.context.ApplicationContext;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.util.ClassUtils;

import java.util.HashMap;
import java.util.Map;

// 基于hashmap实现的内存缓存
// 因为就是服务启动时初始化一次, 无需考虑多线程并发安全问题
public class HashMapCache {
    // 单例: 懒汉模式
    private static final HashMapCache hashMapCache = new HashMapCache();
    private final Map<Object, String> cache = new HashMap<>();
    //将构造器设置为private禁止通过new进行实例化
    private HashMapCache() {

    }
    // 此处实现的是自定义注解的解析逻辑, 将错误码全部加载到内存中
    public void loadCustomException(ApplicationContext applicationContext) {
        Map<String, Object> customExceptionAnnotationMap = applicationContext.getBeansWithAnnotation(CustomExceptionScanAnnotation.class);
        for(Object bean : customExceptionAnnotationMap.values()){
            Class<?> extensionClz = ClassUtils.getUserClass(bean);
            CustomExceptionScanAnnotation customExceptionAnnotation = AnnotationUtils.findAnnotation(extensionClz, CustomExceptionScanAnnotation.class);
            assert customExceptionAnnotation != null;
            String[] scanPackageList = customExceptionAnnotation.exceptionPackageList();
            String[] scanExceptionClassList = customExceptionAnnotation.exceptionClassList();
            System.out.println(scanExceptionClassList, scanPackageList);
        }
    }
    public static HashMapCache getHashMapCache() {
        return hashMapCache;
    }
    // 读取
    public String getCodeMessage(Object code) {
        if (!cache.containsKey(code)) {
            // 不包含指定code
            return code.toString();
        }
        return cache.get(code);
    }

    // 添加
    public void put(Object code, String message) {
        cache.put(code, message);
    }
}
